@@ -399,6 +399,13 @@ _vte_iso2022_ambiguous_width(struct _vte_iso2022_state *state)
 		}
 	}
 
+	/*
+	 * Decide the ambiguous width according to the default region if 
+	 * current locale is UTF-8.
+	 */
+	if (strcmp (codeset, "utf8") == 0 && g_getenv("VTE_CJK_WIDTH") != NULL)
+	  return _vte_iso2022_ambiguous_width_guess ();
+
 	/* Not in the list => not wide. */
 	return 1;
 }
@@ -829,6 +836,7 @@ _vte_iso2022_state_set_codeset(struct _vte_iso2022_state *state,
 	}
 	state->codeset = g_intern_string (codeset);
 	state->conv = conv;
+	state->ambiguous_width = _vte_iso2022_ambiguous_width(state);
 }
 
 const char *
