@@ -19,7 +19,6 @@
 #include "../config.h"
 #include <stdlib.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <string.h>
 #include <glib.h>
 #include <gdk/gdk.h>
@@ -60,6 +59,72 @@ _vte_keysym_name(guint keyval)
 	}
 	return "(unknown)";
 }
+static void
+_vte_keysym_print(guint keyval,
+		GdkModifierType modifiers,
+		gboolean sun_mode,
+		gboolean hp_mode,
+		gboolean legacy_mode,
+		gboolean vt220_mode)
+{
+	g_printerr("Mapping ");
+	if (modifiers & GDK_CONTROL_MASK) {
+		g_printerr("Control+");
+	}
+	if (modifiers & VTE_META_MASK) {
+		g_printerr("Meta+");
+	}
+	if (modifiers & VTE_NUMLOCK_MASK) {
+		g_printerr("NumLock+");
+	}
+	if (modifiers & GDK_SHIFT_MASK) {
+		g_printerr("Shift+");
+	}
+	g_printerr("%s" , _vte_keysym_name(keyval));
+	if (sun_mode|hp_mode|legacy_mode|vt220_mode) {
+		gboolean first = TRUE;
+		g_printerr("(");
+		if (sun_mode) {
+			if (!first) {
+				g_printerr(",");
+			}
+			first = FALSE;
+			g_printerr("Sun");
+		}
+		if (hp_mode) {
+			if (!first) {
+				g_printerr(",");
+			}
+			first = FALSE;
+			g_printerr("HP");
+		}
+		if (legacy_mode) {
+			if (!first) {
+				g_printerr(",");
+			}
+			first = FALSE;
+			g_printerr("Legacy");
+		}
+		if (vt220_mode) {
+			if (!first) {
+				g_printerr(",");
+			}
+			first = FALSE;
+			g_printerr("VT220");
+		}
+		g_printerr(")");
+	}
+}
+#else
+static void
+_vte_keysym_print(guint keyval,
+		GdkModifierType modifiers,
+		gboolean sun_mode,
+		gboolean hp_mode,
+		gboolean legacy_mode,
+		gboolean vt220_mode)
+{
+}
 #endif
 
 enum _vte_cursor_mode {
@@ -204,6 +269,10 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_8[] = {
 	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\177", 1, X_NULL},
 	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, X_NULL},
 };
+static const struct _vte_keymap_entry _vte_keymap_GDK_Minus[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\037", 1, X_NULL},
+	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, X_NULL},
+};
 
 /* Home and End are strange cases because their sequences vary wildly from
  * system to system, or mine's just broken.  But anyway. */
@@ -837,6 +906,7 @@ static const struct _vte_keymap_group {
 	{GDK_6,			_vte_keymap_GDK_6},
 	{GDK_7,			_vte_keymap_GDK_7},
 	{GDK_8,			_vte_keymap_GDK_8},
+	{GDK_minus,		_vte_keymap_GDK_Minus},
 
 	{GDK_Up,		_vte_keymap_GDK_Up},
 	{GDK_Down,		_vte_keymap_GDK_Down},
@@ -932,7 +1002,7 @@ _vte_keymap_map(guint keyval,
 		gssize *normal_length,
 		const char **special)
 {
-	int i;
+	gsize i;
 	const struct _vte_keymap_entry *entries;
 	enum _vte_cursor_mode cursor_mode;
 	enum _vte_keypad_mode keypad_mode;
@@ -948,57 +1018,12 @@ _vte_keymap_map(guint keyval,
 	g_return_if_fail(normal_length != NULL);
 	g_return_if_fail(special != NULL);
 
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
-		fprintf(stderr, "Mapping ");
-		if (modifiers & GDK_CONTROL_MASK) {
-			fprintf(stderr, "Control+");
-		}
-		if (modifiers & VTE_META_MASK) {
-			fprintf(stderr, "Meta+");
-		}
-		if (modifiers & VTE_NUMLOCK_MASK) {
-			fprintf(stderr, "NumLock+");
-		}
-		if (modifiers & GDK_SHIFT_MASK) {
-			fprintf(stderr, "Shift+");
-		}
-		fprintf(stderr, "%s" , _vte_keysym_name(keyval));
-		if (sun_mode || hp_mode || legacy_mode || vt220_mode) {
-			i = 0;
-			fprintf(stderr, "(");
-			if (sun_mode) {
-				if (i > 0) {
-					fprintf(stderr, ",");
-				}
-				i++;
-				fprintf(stderr, "Sun");
-			}
-			if (hp_mode) {
-				if (i > 0) {
-					fprintf(stderr, ",");
-				}
-				i++;
-				fprintf(stderr, "HP");
-			}
-			if (legacy_mode) {
-				if (i > 0) {
-					fprintf(stderr, ",");
-				}
-				i++;
-				fprintf(stderr, "Legacy");
-			}
-			if (vt220_mode) {
-				if (i > 0) {
-					fprintf(stderr, ",");
-				}
-				i++;
-				fprintf(stderr, "VT220");
-			}
-			fprintf(stderr, ")");
-		}
-	}
-#endif
+	_VTE_DEBUG_IF(VTE_DEBUG_KEYBOARD) 
+		_vte_keysym_print(keyval, modifiers,
+				sun_mode,
+				hp_mode,
+				legacy_mode,
+				vt220_mode);
 
 	/* Start from scratch. */
 	*normal = NULL;
@@ -1055,11 +1080,8 @@ _vte_keymap_map(guint keyval,
 		}
 	}
 	if (entries == NULL) {
-#ifdef VTE_DEBUG
-		if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
-			fprintf(stderr, " (ignoring, no map for key).\n");
-		}
-#endif
+		_vte_debug_print(VTE_DEBUG_KEYBOARD,
+				" (ignoring, no map for key).\n");
 		return;
 	}
 
@@ -1103,25 +1125,24 @@ _vte_keymap_map(guint keyval,
 							  hp_mode,
 							  legacy_mode,
 							  vt220_mode,
+							  cursor_mode & cursor_app,
 							  normal,
 							  normal_length);
-#ifdef VTE_DEBUG
-			if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
+			_VTE_DEBUG_IF(VTE_DEBUG_KEYBOARD) {
 				int j;
-				fprintf(stderr, " to '");
+				g_printerr(" to '");
 				for (j = 0; j < *normal_length; j++) {
 					if (((*normal)[j] < 32) ||
 					    ((*normal)[j] >= 127)) {
-						fprintf(stderr, "<0x%02x>",
+						g_printerr("<0x%02x>",
 							(*normal)[j]);
 					} else {
-						fprintf(stderr, "%c",
+						g_printerr("%c",
 							(*normal)[j]);
 					}
 				}
-				fprintf(stderr, "'.\n");
+				g_printerr("'.\n");
 			}
-#endif
 			return;
 		} else {
 			termcap_special = entries[i].special;
@@ -1133,22 +1154,14 @@ _vte_keymap_map(guint keyval,
 				if (strlen(cap) > 0) {
 					/* Save the special string. */
 					*special = entries[i].special;
-#ifdef VTE_DEBUG
-					if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
-						fprintf(stderr, " to \"%s\"",
-							*special);
-					}
-#endif
+					_vte_debug_print(VTE_DEBUG_KEYBOARD,
+							" to \"%s\"", *special);
 				}
 				g_free(cap);
 				if (*special != NULL) {
 					/* Return the special string. */
-#ifdef VTE_DEBUG
-					if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
-						fprintf(stderr,
+					_vte_debug_print(VTE_DEBUG_KEYBOARD,
 							", returning.\n");
-					}
-#endif
 					return;
 				}
 			}
@@ -1179,23 +1192,23 @@ _vte_keymap_map(guint keyval,
 			if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
 				int j;
 #ifdef HAVE_CURSES
-				fprintf(stderr, " via ncurses to '");
+				g_printerr(" via ncurses to '");
 #else
 #ifdef HAVE_TERMCAP
-				fprintf(stderr, " via termcap to '");
+				g_printerr(" via termcap to '");
 #endif
 #endif
 				for (j = 0; j < *normal_length; j++) {
 					if (((*normal)[j] < 32) ||
 					    ((*normal)[j] >= 127)) {
-						fprintf(stderr, "<0x%02x>",
+						g_printerr("<0x%02x>",
 							(*normal)[j]);
 					} else {
-						fprintf(stderr, "%c",
+						g_printerr("%c",
 							(*normal)[j]);
 					}
 				}
-				fprintf(stderr, "', returning.\n");
+				g_printerr("', returning.\n");
 			}
 #endif
 			return;
@@ -1203,11 +1216,8 @@ _vte_keymap_map(guint keyval,
 	}
 #endif
 
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
-		fprintf(stderr, " (ignoring, no match for modifier state).\n");
-	}
-#endif
+	_vte_debug_print(VTE_DEBUG_KEYBOARD,
+			" (ignoring, no match for modifier state).\n");
 }
 
 gboolean
@@ -1319,6 +1329,31 @@ _vte_keymap_key_gets_modifiers(guint keyval)
 	return fkey;
 }
 
+/* Prior and Next are ommitted for the SS3 to CSI switch below */
+gboolean
+is_cursor_key(guint keyval)
+{
+	switch (keyval) {
+	case GDK_Home:
+	case GDK_Left:
+	case GDK_Up:
+	case GDK_Right:
+	case GDK_Down:
+	case GDK_End:
+
+	case GDK_KP_Home:
+	case GDK_KP_Left:
+	case GDK_KP_Up:
+	case GDK_KP_Right:
+	case GDK_KP_Down:
+	case GDK_KP_End:
+		return TRUE;
+	default:
+		return FALSE;
+	}
+}
+
+
 void
 _vte_keymap_key_add_key_modifiers(guint keyval,
 				  GdkModifierType modifiers,
@@ -1326,6 +1361,7 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 				  gboolean hp_mode,
 				  gboolean legacy_mode,
 				  gboolean vt220_mode,
+				  gboolean cursor_app_mode,
 				  char **normal,
 				  gssize *normal_length)
 {
@@ -1382,6 +1418,15 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 	nnormal = g_malloc0(*normal_length + 4);
 	memcpy(nnormal, *normal, *normal_length);
 	if (strlen(nnormal) > 1) {
+		/* SS3 should have no modifiers so make it CSI instead. See
+		 * http://cvsweb.xfree86.org/cvsweb/xc/programs/xterm/input.c.diff?r1=3.57&r2=3.58
+		 */
+		if (cursor_app_mode &&
+			g_str_has_prefix(nnormal, _VTE_CAP_SS3)
+			&& is_cursor_key(keyval)) {
+			nnormal[1] = '[';
+		}
+
 		/* Get the offset of the last character. */
 		offset = strlen(nnormal) - 1;
 		if (g_ascii_isdigit(nnormal[offset - 1])) {
@@ -1392,7 +1437,7 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 			nnormal[offset + 0] = ';';
 			*normal_length += 2;
 		} else {
-#if 0
+#if 1
 			/* Stuff a "1", a semicolon and the modifier in right
 			 * before that last character, matching Xterm. */
 			nnormal[offset + 3] = nnormal[offset];
