@@ -55,6 +55,8 @@ G_BEGIN_DECLS
 #define VTE_PAD_WIDTH			1
 #define VTE_TAB_WIDTH			8
 #define VTE_LINE_WIDTH			1
+#define VTE_ROWS			24
+#define VTE_COLUMNS			80
 #define VTE_LEGACY_COLOR_SET_SIZE	8
 #define VTE_COLOR_PLAIN_OFFSET		0
 #define VTE_COLOR_BRIGHT_OFFSET		8
@@ -80,10 +82,12 @@ G_BEGIN_DECLS
 #define VTE_REGCOMP_FLAGS		REG_EXTENDED
 #define VTE_REGEXEC_FLAGS		0
 #define VTE_INPUT_CHUNK_SIZE		0x2000
+#define VTE_MAX_INPUT_READ		0x1000
 #define VTE_INVALID_BYTE		'?'
 #define VTE_DISPLAY_TIMEOUT		10
-#define VTE_UPDATE_TIMEOUT		10
-#define VTE_UPDATE_REPEAT_TIMEOUT	25
+#define VTE_UPDATE_TIMEOUT		15
+#define VTE_UPDATE_REPEAT_TIMEOUT	30
+#define VTE_MAX_PROCESS_TIME		100
 #define VTE_CELL_BBOX_SLACK		1
 
 /* The structure we use to hold characters we're supposed to display -- this
@@ -91,27 +95,29 @@ G_BEGIN_DECLS
 struct vte_charcell {
 	gunichar c;		/* The Unicode character. */
 
-	guint32 columns: 2;	/* Number of visible columns (as determined
-				   by g_unicode_iswide(c)). */
-	guint32 fore: 9;	/* Indices in the color palette for the */
-	guint32 back: 9;	/* foreground and background of the cell. */
+	struct vte_charcell_attr {
+		guint32 columns: 2;	/* Number of visible columns
+					   (as determined
+					   by g_unicode_iswide(c)). */
+		guint32 fore: 9;	/* Index into color palette */
+		guint32 back: 9;	/* Index into color palette. */
 
-	guint32 fragment: 1;	/* The nth fragment of a wide character. */
-	guint32 standout: 1;	/* Single-bit attributes. */
-	guint32 underline: 1;
-	guint32 strikethrough: 1;
+		guint32 fragment: 1;	/* A continuation cell. */
+		guint32 standout: 1;	/* Single-bit attributes. */
+		guint32 underline: 1;
+		guint32 strikethrough: 1;
 
-	guint32 reverse: 1;
-	guint32 blink: 1;
-	guint32 half: 1;
-	guint32 bold: 1;
+		guint32 reverse: 1;
+		guint32 blink: 1;
+		guint32 half: 1;
+		guint32 bold: 1;
 
-	guint32 invisible: 1;
-	guint32 protect: 1;
-	guint32 alternate: 1;
-	guint32 empty : 1;
+		guint32 invisible: 1;
+		guint32 protect: 1;
+		guint32 alternate: 1;
 
-	/* no more bits left.  any addition will enlarge the struct */
+		/* 31 bits */
+	} attr;
 };
 
 /* A match regex, with a tag. */
@@ -169,6 +175,7 @@ struct _VteTerminalPrivate {
 	int pty_master;			/* pty master descriptor */
 	GIOChannel *pty_input;		/* master input watch */
 	guint pty_input_source;
+	gboolean pty_input_active;
 	GIOChannel *pty_output;		/* master output watch */
 	guint pty_output_source;
 	GPid pty_pid;			/* pid of child using pty slave */
@@ -187,6 +194,8 @@ struct _VteTerminalPrivate {
 	GSList *update_regions;
 	gboolean invalidated_all;	/* pending refresh of entire terminal */
 	GList *active;                  /* is the terminal processing data */
+	glong input_bytes;
+	glong max_input_bytes;
 
 	/* Output data queue. */
 	struct _vte_buffer *outgoing;	/* pending input characters */
@@ -268,12 +277,11 @@ struct _VteTerminalPrivate {
 	gboolean smooth_scroll;
 	GHashTable *tabstops;
 	gboolean text_modified_flag;
-	glong text_inserted_count;
-	glong text_deleted_count;
+	gboolean text_inserted_flag;
+	gboolean text_deleted_flag;
 
 	/* Scrolling options. */
 	gboolean scroll_background;
-	long scroll_lock_count;
 	gboolean scroll_on_output;
 	gboolean scroll_on_keystroke;
 	long scrollback_lines;
@@ -302,10 +310,12 @@ struct _VteTerminalPrivate {
 	char *match_contents;
 	GArray *match_attributes;
 	GArray *match_regexes;
-	int match_previous;
+	char *match;
+	int match_tag;
 	struct {
 		long row, column;
 	} match_start, match_end;
+	gboolean show_match;
 
 	/* Data used when rendering the text which does not require server
 	 * resources and which can be kept after unrealizing. */
@@ -345,17 +355,21 @@ struct _VteTerminalPrivate {
 	gboolean adjustment_changed_pending;
 	gboolean adjustment_value_changed_pending;
 
+	gboolean cursor_moved_pending;
+	gboolean contents_changed_pending;
+
 	/* window name changes */
 	gchar *window_title_changed;
 	gchar *icon_title_changed;
 
 	/* Background images/"transparency". */
+	guint root_pixmap_changed_tag;
 	gboolean bg_update_pending;
 	gboolean bg_transparent;
 	GdkPixbuf *bg_pixbuf;
 	char *bg_file;
 	GdkColor bg_tint_color;
-	long bg_saturation;	/* out of VTE_SATURATION_MAX */
+	guint16 bg_saturation;	/* out of VTE_SATURATION_MAX */
 	guint16 bg_opacity;
 
 	/* Key modifiers. */
@@ -368,7 +382,7 @@ struct _VteTerminalPrivate {
 };
 
 
-VteRowData *_vte_terminal_ensure_cursor(VteTerminal *terminal, gboolean current);
+VteRowData *_vte_terminal_ensure_row(VteTerminal *terminal);
 void _vte_terminal_set_pointer_visible(VteTerminal *terminal, gboolean visible);
 void _vte_invalidate_all(VteTerminal *terminal);
 void _vte_invalidate_cells(VteTerminal *terminal,
@@ -380,16 +394,12 @@ VteRowData * _vte_new_row_data(VteTerminal *terminal);
 VteRowData * _vte_new_row_data_sized(VteTerminal *terminal, gboolean fill);
 VteRowData * _vte_reset_row_data (VteTerminal *terminal, VteRowData *row, gboolean fill);
 void _vte_terminal_adjust_adjustments(VteTerminal *terminal);
-void _vte_terminal_emit_contents_changed(VteTerminal *terminal);
+void _vte_terminal_queue_contents_changed(VteTerminal *terminal);
 void _vte_terminal_emit_text_deleted(VteTerminal *terminal);
 void _vte_terminal_emit_text_inserted(VteTerminal *terminal);
-void _vte_terminal_insert_char(VteTerminal *terminal, gunichar c,
+gboolean _vte_terminal_insert_char(VteTerminal *terminal, gunichar c,
 			       gboolean force_insert_mode,
-			       gboolean invalidate_cells,
-			       gboolean paint_cells,
-			       gboolean ensure_after,
-			       gint forced_width);
-void _vte_terminal_match_contents_clear(VteTerminal *terminal);
+			       gboolean invalidate_cells);
 void _vte_terminal_scroll_region(VteTerminal *terminal,
 				 long row, glong count, glong delta);
 void _vte_terminal_set_default_attributes(VteTerminal *terminal);
